# Custom migration data type
# Based on papercli schema.rb from: https://github.com/thoughtbot/paperclip => /lib/paperclip/schema.rb
module PaperCropper
  module Schema
    COLUMNS = {
      crop_x: :integer,
      crop_y: :integer,
      crop_width: :integer,
      crop_height: :integer,
      crop_ratio: [:decimal, {precision: 8, scale: 4}]
    }

    #
    # #########################################################################
    module Statements
      def add_crop_attachment(table_name, *attachment_names)
        raise ArgumentError, "Please specify attachment name in your add_attachment call in your migration." if attachment_names.empty?

        options = attachment_names.extract_options!

        attachment_names.each do |attachment_name|
          COLUMNS.each_pair do |column_name, column_definition|
            column_type, default_options = column_definition
            custom_options = options.merge(options[column_name.to_sym] || {})
            column_options = (default_options || {}).merge(custom_options)
            add_column(table_name, "#{attachment_name}_#{column_name}", column_type, column_options)
          end
        end
      end

      def remove_crop_attachment(table_name, *attachment_names)
        raise ArgumentError, "Please specify attachment name in your remove_attachment call in your migration." if attachment_names.empty?

        attachment_names.each do |attachment_name|
          COLUMNS.keys.each do |column_name|
            remove_column(table_name, "#{attachment_name}_#{column_name}")
          end
        end
      end
    end

    #
    # #########################################################################
    module TableDefinition
      def crop_attachment(*attachment_names)
        options = attachment_names.extract_options!
        attachment_names.each do |attachment_name|
          COLUMNS.each_pair do |column_name, column_definition|
            column_type, default_options = column_definition
            custom_options = options.merge(options[column_name.to_sym] || {})
            column_options = (default_options || {}).merge(custom_options)
            column("#{attachment_name}_#{column_name}", column_type, column_options)
          end
        end
      end
    end

    #
    # #########################################################################
    module CommandRecorder
      def add_attachment(*args)
        record(:add_attachment, args)
      end

      private

      def invert_add_attachment(args)
        [:remove_attachment, args]
      end
    end
  end
end



# ActiveRecord support
# #########################################################################
ActiveRecord::ConnectionAdapters::Table.send :include, PaperCropper::Schema::TableDefinition
ActiveRecord::ConnectionAdapters::TableDefinition.send :include, PaperCropper::Schema::TableDefinition
ActiveRecord::ConnectionAdapters::AbstractAdapter.send :include, PaperCropper::Schema::Statements
ActiveRecord::Migration::CommandRecorder.send :include, PaperCropper::Schema::CommandRecorder
